/*
	$Id: AnnotatableProxy.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

/**
	This interface is implemented by items that can be used to hold annotations 
	that are intended for other items. This is for example used to enable plug-in
	configurations to hold annotation that can be copied to other items when a 
	job using that configuration is executed. The {@link net.sf.basedb.plugins.RawDataFlatFileImporter}
	supports this feature and can be used to automatically annotate a raw bioassay
	when raw data is imported.
		
	@author Nicklas
	@version 2.5
	@see net.sf.basedb.core.plugin.AnnotationSetterPlugin
	@see AnnotationSet
	@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/
public interface AnnotatableProxy
	extends Annotatable
{

	/**
		Get the annotation types that can be proxied. This is, for example,
		in the case of the {@link net.sf.basedb.plugins.RawDataFlatFileImporter} 
		all annotation types that can be used on raw bioassays.
		@return A query, or null if no annotation types can be used
	*/
	public ItemQuery<AnnotationType> getAnnotationTypes();

	/**
		A help message that should tell users how the proxied annotations
		are used.
		@return A message or null
	*/
	public String getAnnotationMessage();
	
}
