/**
	$Id: VersionedParameter.java 5818 2011-10-21 11:15:41Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.io.Serializable;

/**
 	The key to a plugin configuration parameter. The parameters are
 	versioned because we need to remember exactly which parameters
 	that was passed to a job.

	@author nicklas
	@version 2.0
	@see <a href="../../../../../../html/developer/api/data_api.plugins.html">Developer documentation: Plug-ins, jobs and job agents</a>
	@base.modified $Date: 2011-10-21 13:15:41 +0200 (Fri, 21 Oct 2011) $
*/
public class VersionedParameter
	implements Serializable
{
	/**
		
	*/
	private static final long serialVersionUID = 840111442332381385L;

	/**
		The name of the parameter.
	*/
	private String name;
	
	/**
		The version of the parameter.
	*/
	private int version;
	
	/**
		Used by Hibernate to create an instance.
	*/
	VersionedParameter()
	{}
	
	/**
		Create a new instance.
	*/
	public VersionedParameter(String name, int version)
	{
		this.name = name;
		this.version = version;
	}
	
	/**
		Get the parameter name.
		@hibernate.property column="`name`" not-null="true" type="string" length="255"
	*/
	public String getName()
	{
		return name;
	}
	void setName(String name)
	{
		this.name = name;
	}
	
	/**
		Get the parameter version.
		@hibernate.property column="`parameter_version`" type="int" not-null="true"
	*/
	public int getVersion()
	{
		return version;
	}
	void setVersion(int version)
	{
		this.version = version;
	}
	
	/**
		Check if this object is equal to another <code>VersionedParameter</code>
		object. They are considered to be the same if the have the same
		name and version.
	*/
	public boolean equals(Object o)
	{
		if ((o == null) || (getClass() != o.getClass())) return false;
		VersionedParameter other = (VersionedParameter)o;
		return this.name.equals(other.name) && this.version == other.version;
	}
	
	/**
		Calculate the hash code for the object.
	*/
	public int hashCode()
	{
		return version + (name == null ? 0 : name.hashCode());
	}
	/**
		Convert the value to a string.
	*/
	public String toString()
	{
		return name + "[" + version + "]";
	}

}
