/**
	$Id: AnnotationTypeFilter.java 5131 2009-10-14 07:44:14Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.snapshot;

import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.util.filter.Filter;

/**
	Filter implementation that finds all annotation snapshots 
	for a given annotation type.

	@author Nicklas
	@version 2.14
	@base.modified $Date: 2009-10-14 09:44:14 +0200 (Wed, 14 Oct 2009) $
*/
public class AnnotationTypeFilter
	implements Filter<AnnotationSnapshot>
{
	private int annotationTypeId;
	
	/**
		Create a filter that matches all annotation types.
	*/
	public AnnotationTypeFilter()
	{}
	
	/**
		Create a filter that matches a specfic annotation type.
	*/
	public AnnotationTypeFilter(AnnotationType at)
	{
		this.annotationTypeId = at.getId();
	}
	/*
		From the Filter interface
		-------------------------
	*/
	@Override
	public boolean evaluate(AnnotationSnapshot snapshot)
	{
		return snapshot.getAnnotationTypeId() == annotationTypeId || annotationTypeId == 0;
	}
	// -------------------------

	/**
		Set the annotation type this filter should match.
		@param at A specific annotation type, or null to match any annotation type
	*/
	public void setAnnotationType(AnnotationType at)
	{
		this.annotationTypeId = at == null ? 0 : at.getId();
	}
	
}
