/*
	$Id: TestAnnotationType.java 5422 2010-09-22 08:01:44Z nicklas $
	
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

import java.util.Date;
import java.util.Set;

import net.sf.basedb.core.*;
import java.util.Arrays;

public class TestAnnotationType
{
	static boolean ok = true;

	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing annotation types");
		write_header();
		
		int celsiusId = TestUnit.test_load_by_symbol(SystemItems.getId(Quantity.TEMPERATURE), "C");
		int meterId = TestUnit.test_load_by_symbol(SystemItems.getId(Quantity.LENGTH), "m");
		
		// Standard tests: create, load, list
		int id_int = test_create(null, Type.INT, 0, 10, null, Item.SAMPLE, 1, null, true);
		int id_long = test_create(null, Type.LONG, 0, 123456789123456l, null, Item.SAMPLE, 1, null, true);
		int id_float = test_create("Temperature", Type.FLOAT, celsiusId, 10.1, null, Item.SAMPLE, 1, null, false);
		int id_double = test_create("Size", Type.DOUBLE, meterId, 8.13459187745, null, Item.SAMPLE, 1, null, false);
		int id_date = test_create(null, Type.DATE, 0, new Date(), null, Item.SAMPLE, 1, null, false);
		int id_timestamp = test_create(null, Type.TIMESTAMP, 0, new Date(), null, Item.SAMPLE, 1, null, false);
		int id_text = test_create(null, Type.TEXT, 0, "default value", null, Item.SAMPLE, 1, null, false);
		int id_string = test_create(null, Type.STRING, 0, "default value", null, Item.SAMPLE, 1, null, false);
		int id_boolean = test_create(null, Type.BOOLEAN, 0, true, null, Item.SAMPLE, 1, null, false);
		int id_enum = test_create(null, Type.INT, 0, 3, null, Item.SAMPLE, 1, new Object[] { 1, 2, 3, 4, 5 }, false);
		int id2_enum = test_create(null, Type.STRING, 0, "one", null, Item.SAMPLE, 1, new String[] { "one", "two ", "three" }, false);
		test_load(id_int);
		test_load(id_long);
		test_load(id_float);
		test_load(id_double);
		test_load(id_date);
		test_load(id_timestamp);
		test_load(id_text);
		test_load(id_string);
		test_load(id_boolean);
		test_load(id_enum);
		test_load(id2_enum);
		
		test_list(-1);

		test_find_value(id2_enum, "one", false, false, "one");
		test_find_value(id2_enum, "one ", false, false, null);
		test_find_value(id2_enum, "one ", false, true, "one");
		test_find_value(id2_enum, "ONE", false, false, null);
		test_find_value(id2_enum, "ONE", true, false, "one");
		test_find_value(id2_enum, "ONE ", true, false, null);
		test_find_value(id2_enum, "ONE ", true, true, "one");
		test_find_value(id2_enum, "two", false, false, null);
		test_find_value(id2_enum, "two ", false, false, "two ");
		test_find_value(id2_enum, "two", false, true, "two ");
		test_find_value(id2_enum, "TWO", false, false, null);
		test_find_value(id2_enum, "TWO ", false, false, null);
		test_find_value(id2_enum, "TWO ", true, false, "two ");
		test_find_value(id2_enum, "TWO", true, true, "two ");
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();

		// Standard test: Delete
		test_delete(id_int);
		test_delete(id_long);
		test_delete(id_float);
		test_delete(id_double);
		test_delete(id_date);
		test_delete(id_timestamp);
		test_delete(id_text);
		test_delete(id_string);
		test_delete(id_boolean);
		test_delete(id_enum);
		write("++Testing annotation types "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}
	
	static int test_create(String name, Type type, int unitId, 
			Object defaultValue, Number maxValue, Item itemType, int multiplicity, Object[] enumValues, boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.ANNOTATIONTYPE)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			AnnotationType at = AnnotationType.getNew(dc, type);
			if(setAll)
			{
				at.setName( "Test annotation type: "+type);
				at.setDescription("Added at "+new Date());
				at.setRequiredForMiame(true);
				at.setHeight(10);
				at.setWidth(60);
				at.setExternalId("my.external.annotationtype");
			}
			at.setMultiplicity(multiplicity);
			if (name != null) at.setName(name);
			if (unitId != 0)
			{
				at.setDefaultUnit(Unit.getById(dc, unitId));
			}
			if (enumValues != null)
			{
				at.setEnumeration(true);
				at.setValues(Arrays.asList(enumValues));
				at.setDisplayAsList(true);
			}
			if (defaultValue != null) at.setDefaultValue(defaultValue.toString());
			if (maxValue != null)
			{
				if (type.isNumerical()) 
				{
					at.setMinValueLong(0l);
					at.setMaxValueLong(maxValue.longValue());
				}
				else if (type == Type.STRING) 
				{
					at.setMaxLength(maxValue.intValue());
				}
			}
			at.enableForItem(itemType);
			dc.saveItem(at);
			dc.commit();
			dc = TestUtil.getDbControl();
			dc.reattachItem(at, false);
			id = at.getId();
			write_item(0, at);
			write("--Create annotation type OK");
		}
		catch (Throwable ex)
		{
			write("--Create annotation type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			AnnotationType at = AnnotationType.getById(dc, id);
			write_item(0, at);
			write("--Load annotation type OK");
		}
		catch (Throwable ex)
		{
			write("--Load annotation type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<AnnotationType> l = AnnotationType.getQuery(null).list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List annotation types OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List annotation types FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			AnnotationType at = AnnotationType.getById(dc, id);
			dc.deleteItem(at);
			Set<ItemProxy> using = at.getUsingItems();
			if (using.size() > 0) 
			{
			   throw new BaseException(at + " is used by " + using);
			}
			dc.commit();
			write("--Delete annotation type OK");
		}
		catch (Throwable ex)
		{
			write("--Delete annotation type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tValue type\tDefault unit\tMultiplicity\tExternal ID\tWidth\tHeight\tMiame\tDefault value\tItem types\tEnumeration values");
			write("-- \t-- \t--------- \t-----------\t----------\t------------\t------------\t-----------\t-----\t------\t-----\t-------------\t----------\t------------------");
		}
	}
	static void write_item(int i, AnnotationType at)
		throws BaseException
	{
		if (!TestUtil.getSilent()) 
			write(i+":\t"+at.getId()+"\t"+at.getName()+"\t"+at.getDescription()+"\t"+at.getValueType()+
			"\t" + at.getDefaultUnit()+ 
			"\t"+at.getMultiplicity()+"\t"+at.getExternalId()+"\t"+at.getWidth()+"\t"+at.getHeight()+"\t"+at.isRequiredForMiame()+
			"\t"+at.getDefaultValue()+"\t"+at.getEnabledItems()+"\t"+at.getValues());
	}
	static void write(String message)
	{
		System.out.println(message);
	}
	
	static void test_find_value(int id, String value, boolean ignoreCase, boolean trim, String expected)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			AnnotationType at = AnnotationType.getById(dc, id);
			String found = at.findValue(value, ignoreCase, trim);
			if ((expected != null && !expected.equals(found)) || (expected == null && found != null))
			{
				throw new Exception("Looking for '" + value + "'; found '" + found + 
					"'; expected '" + expected + "'");
			}
			write("--Find annotation value OK; looking for '" + value + "'; found '" + found + "'");
		}
		catch (Throwable ex)
		{
			write("--Find annotation value FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}	
	
}
