/*
	$Id: TestBioMaterialList.java 5340 2010-05-10 06:15:00Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/


import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BioMaterial;
import net.sf.basedb.core.BioMaterialList;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemProxy;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.Permission;

public class TestBioMaterialList
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing biomaterial lists");
		write_header();
		// Standard tests: create, load, list
		int id1 = test_create(Item.BIOSOURCE, "net.sf.basedb.clients.test.bmlist1", true);
		int id2 = test_create(Item.BIOSOURCE, "net.sf.basedb.clients.test.bmlist2", true);
		int id3 = test_create(Item.BIOSOURCE, "net.sf.basedb.clients.test.bmlist3", true);
		int id4 = test_create(Item.BIOSOURCE, "net.sf.basedb.clients.test.bmlist4", true);
		int id5 = test_create(Item.BIOSOURCE, "net.sf.basedb.clients.test.bmlist5", true);
		int id6 = test_create(Item.BIOSOURCE, "net.sf.basedb.clients.test.bmlist6", true);
		test_load(id1);
		test_list(-1);

		// Create items to use in the lists
		int bs1 = TestBioSource.test_create("bs.1", false);
		int bs2 = TestBioSource.test_create("bs.2", false);
		int bs3 = TestBioSource.test_create("bs.3", false);
		int bs4 = TestBioSource.test_create("bs.4", false);
		int bs5 = TestBioSource.test_create("bs.5", false);
		
		test_add_members(id1, bs1, bs2, bs3);
		test_list_members(id1, 3);
		test_add_members(id2, bs3, bs4, bs5); // One in common with 'id1'
		
		test_union(id3, 5, id1, id2); // 'id3' is now the union of 'id1' and 'id2'
		test_intersect(id4, 1, id1, id2); // 'id4' is now the intersection of 'id1' and 'id2'
		test_remove(id5, id3, 4, id4); // 'id5' is now the complement of 'id3' to 'id4'
		test_retain(id6, id2, 1, id1); // 'id6' is now the intersection on 'id1' and 'id2'
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_remove_member(id1, bs1);
		test_remove_member(id1, bs2);
		test_list_members(id1, 1);
		
		// Standard test: Delete
		test_delete(id1);
		test_delete(id2);
		test_delete(id3);
		test_delete(id4);
		test_delete(id5);
		test_delete(id6);
		TestBioSource.test_delete(bs1);
		TestBioSource.test_delete(bs2);
		TestBioSource.test_delete(bs3);
		TestBioSource.test_delete(bs4);
		TestBioSource.test_delete(bs5);
		write("++Testing biomaterial lists "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(Item memberType, String externalId, boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.BIOMATERIALLIST)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioMaterialList bl = BioMaterialList.getNew(dc, memberType);
			bl.setExternalId(externalId);
			if (setAll)
			{
				bl.setName("Test biomaterial list");
				bl.setDescription("Added at "+new Date());
			}
			dc.saveItem(bl);
			dc.commit();
			id = bl.getId();
			write_item(0, bl);
			write("--Create biomaterial list OK");
		}
		catch (Throwable ex)
		{
			write("--Create biomaterial list FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioMaterialList bl = BioMaterialList.getById(dc, id);
			write_item(0, bl);
			write("--Load biomaterial list OK");
		}
		catch (Throwable ex)
		{
			write("--Load biomaterial list FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}


	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemQuery<BioMaterialList> q = BioMaterialList.getQuery();
			ItemResultList<BioMaterialList> l = q.list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List biomaterial lists OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List biomaterial lists FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioMaterialList bl = BioMaterialList.getById(dc, id);
			dc.deleteItem(bl);
			Set<ItemProxy> using = bl.getUsingItems();
			if (using.size() > 0) 
			{
			   throw new BaseException(bl + " is used by " + using);
			}
			dc.commit();
			write("--Delete biomaterial list OK");
		}
		catch (Throwable ex)
		{
			write("--Delete biomaterial list FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tExternal ID\tSize");
			write("-- \t-- \t--------- \t-----------\t-----------\t----");
		}
	}
	static void write_item(int i, BioMaterialList bl)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+bl.getId()+"\t"+bl.getName()+"\t"+
			bl.getDescription()+"\t"+bl.getExternalId()+"\t"+bl.getSize());
	}
	static void write_item(int i, BioMaterial bm)
	throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+bm.getId()+"\t"+bm.getName()+"\t"+
			bm.getType());
	}
	static void write(String message)
	{
		System.out.println(message);
	}

	static void test_add_members(int id, int... itemIds)
	{
		if (id == 0 || itemIds == null || itemIds.length == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioMaterialList bl = BioMaterialList.getById(dc, id);
			int oldSize = bl.getSize();
			for (int iid : itemIds)
			{
				BioMaterial bm = (BioMaterial)bl.getMemberType().getById(dc, iid);
				bl.add(bm);
			}
			dc.commit();
			
			dc = TestUtil.getDbControl();
			bl = BioMaterialList.getById(dc, id);
			int newSize = bl.getSize();
			if (newSize != oldSize + itemIds.length)
			{
				throw new BaseException("new size=" + newSize + "; old size="+oldSize);
			}
			write("--Add biomaterial to list OK (new size=" + newSize + ")");
		}
		catch (Throwable ex)
		{
			write("--Add biomaterial to list FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_remove_member(int id, int itemId)
	{
		if (id == 0 || itemId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioMaterialList bl = BioMaterialList.getById(dc, id);
			int oldSize = bl.getSize();
			BioMaterial bm = (BioMaterial)bl.getMemberType().getById(dc, itemId);
			bl.remove(bm);
			dc.commit();
			dc = TestUtil.getDbControl();
			bl = BioMaterialList.getById(dc, id);
			int newSize = bl.getSize();
			if (newSize != oldSize - 1)
			{
				throw new BaseException("new size=" + newSize + "; old size="+oldSize);
			}
			write("--Remove biomaterial from list OK (new size=" + newSize + ")");
		}
		catch (Throwable ex)
		{
			write("--Remove biomaterial from list FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_list_members(int id, int expectedResults)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioMaterialList bl = BioMaterialList.getById(dc, id);
			ItemQuery<? extends BioMaterial> query = bl.getMemberBioMaterials();
			ItemResultList<? extends BioMaterial> l = query.list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List members in biomaterial list OK (" + l.size() + ")");
		}
		catch (Throwable ex)
		{
			write("--List members in biomaterial list FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_union(int toId, int expectedSize, int... fromId)
	{
		if (toId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioMaterialList to = BioMaterialList.getById(dc, toId);
			List<BioMaterialList> from = new ArrayList<BioMaterialList>(fromId.length);
			for (int rlId : fromId)
			{
				from.add(BioMaterialList.getById(dc, rlId));
			}
			to.addUnion(from);
			dc.commit();
			if (expectedSize != to.getSize())
			{
				throw new BaseException("Incorrect size; expected=" + expectedSize + "; actual=" + to.getSize());
			}
			write("--Create union OK (size=" + expectedSize + ")");
		}
		catch (Throwable ex)
		{
			write ("--Create union FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_intersect(int toId, int expectedSize, int... fromId)
	{
		if (toId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioMaterialList to = BioMaterialList.getById(dc, toId);
			List<BioMaterialList> from = new ArrayList<BioMaterialList>(fromId.length);
			for (int rlId : fromId)
			{
				from.add(BioMaterialList.getById(dc, rlId));
			}
			to.addIntersection(from);
			dc.commit();
			if (expectedSize != to.getSize())
			{
				throw new BaseException("Incorrect size; expected=" + expectedSize + "; actual=" + to.getSize());
			}
			write("--Create intersection OK (size=" + expectedSize + ")");
		}
		catch (Throwable ex)
		{
			write ("--Create intersection FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_remove(int toId, int baseId, int expectedSize, int... complementId)
	{
		if (toId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioMaterialList to = BioMaterialList.getById(dc, toId);
			to.addAll(BioMaterialList.getById(dc, baseId));
			List<BioMaterialList> from = new ArrayList<BioMaterialList>(complementId.length);
			for (int rlId : complementId)
			{
				from.add(BioMaterialList.getById(dc, rlId));
			}
			to.removeUnion(from);
			dc.commit();
			if (expectedSize != to.getSize())
			{
				throw new BaseException("Incorrect size; expected=" + expectedSize + "; actual=" + to.getSize());
			}
			write("--Remove items OK (size=" + expectedSize + ")");
		}
		catch (Throwable ex)
		{
			write ("--Remove items FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_retain(int toId, int baseId, int expectedSize, int... complementId)
	{
		if (toId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			BioMaterialList to = BioMaterialList.getById(dc, toId);
			to.addAll(BioMaterialList.getById(dc, baseId));
			List<BioMaterialList> from = new ArrayList<BioMaterialList>(complementId.length);
			for (int rlId : complementId)
			{
				from.add(BioMaterialList.getById(dc, rlId));
			}
			to.retainUnion(from);
			dc.commit();
			if (expectedSize != to.getSize())
			{
				throw new BaseException("Incorrect size; expected=" + expectedSize + "; actual=" + to.getSize());
			}
			write("--Retain items OK (size=" + expectedSize + ")");
		}
		catch (Throwable ex)
		{
			write ("--Retain items FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	
}

