/*
	$Id: TestClient.java 5827 2011-10-26 10:51:52Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;

import java.util.Date;
import java.util.Set;

public class TestClient
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing client");
		write_header();
		// Standard tests: create, load, list
		int id = test_create("net.sf.basedb.clients.testcreate", true);
		int id2 = test_create("net.sf.basedb.clients.testcreate2", false);
		test_load(id);
		test_list(-1);

		// Extra tests: log in and log out with the new client
		test_login_logout(id);

		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		// Standard test: Delete
		test_delete(id);
		test_delete(id2);
		write("++Testing client "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(String externalId, boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.CLIENT)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Client c = Client.getNew(dc, externalId);
			if (setAll)
			{
				c.setName("Test client");
				c.setDescription("Added at "+new Date());
			}
			dc.saveItem(c);
			dc.commit();
			id = c.getId();
			write_item(0, c);
			write("--Create client OK");
		}
		catch (Throwable ex)
		{
			write("--Create client FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Client c = Client.getById(dc, id);
			write_item(0, c);
			write("--Load client OK");
		}
		catch (Throwable ex)
		{
			write("--Load client FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}


	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemQuery<Client> q = Client.getQuery();
			ItemResultList<Client> l = q.list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List clients OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List clients FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}


	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Client c = Client.getById(dc, id);
			dc.deleteItem(c);
			Set<ItemProxy> using = c.getUsingItems();
			if (using.size() > 0) 
			{
			   throw new BaseException(c + " is used by " + using);
			}
			dc.commit();
			write("--Delete client OK");
		}
		catch (Throwable ex)
		{
			write("--Delete client FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tExternal ID");
			write("-- \t-- \t--------- \t-----------\t-----------");
		}
	}
	static void write_item(int i, Client c)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+c.getId()+"\t"+c.getName()+"\t"+
			c.getDescription()+"\t"+c.getExternalId());
	}
	static void write(String message)
	{
		System.out.println(message);
	}

	static void test_login_logout(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Client c = Client.getById(dc, id);
			SessionControl sc = Application.newSessionControl(c.getExternalId(), null, null);
			sc.login(TestUtil.getLogin(), TestUtil.getPassword(), "Running test program");
			sc.logout();
			write("--Login/logout OK");
		}
		catch (Throwable ex)
		{
			write("--Login/logout FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}



}

