/*
	$Id: TestMedianRatioNormalization.java 5319 2010-04-20 08:02:16Z nicklas $

	Copyright (C) 2006 Johan Enell, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.PluginConfigurationRequest;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.PluginResponse;
import net.sf.basedb.core.plugin.Response;
import net.sf.basedb.plugins.MedianRatioNormalization;

public class TestMedianRatioNormalization
{
	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		boolean wasSetup = TestAnalyzeUtil.setupExperiment();
		write("++Testing MedianRatio normalization");
		try
		{
			int experimentId = TestAnalyzeUtil.getItemId("experiment.genepix");
			int rootBasId = TestAnalyzeUtil.getItemId("bioassayset.root");
			if (experimentId == 0 || rootBasId == 0)
			{
				ok = false;
				write("Test experiment was not setup correctly.");
				return ok;
			}
	
			int jobId = test_create_job(experimentId, rootBasId, "Median-ratio normalized", 100, 5, 5, 1);
			ok &= TestJob.test_execute(jobId, false);
			
			int normalizedBasId = TestAnalyzeUtil.getBioAssaySetIdByName(experimentId, "Median-ratio normalized");
			TestAnalyzeUtil.setItemId("bioassayset.median-ratio", normalizedBasId);
		}
		finally
		{
			// Cleanup
			if (wasSetup) TestAnalyzeUtil.cleanupExperiment();
		}
		write("++Testing Median-ratio normalization "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}
	
	
	static int test_create_job(int experimentId, int bioAssaySetId, String childName, 
		float minIntensity, float lowExclude, float highExclude, int blockGroup)
	{
		if (experimentId == 0 || bioAssaySetId == 0 || 
			!TestUtil.hasPermission(Permission.CREATE, Item.JOB)) 
		{
			return 0;
		}
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();

			Experiment e = Experiment.getById(dc, experimentId);
			BioAssaySet bioAssaySet = BioAssaySet.getById(dc, bioAssaySetId);
						
			PluginDefinition plugin = PluginDefinition.getByClassName(dc, MedianRatioNormalization.class.getName());
			Job j = Job.getNew(dc, plugin, null, e);
			j.setName("Normalizing using Media-ratio");
			dc.getSessionControl().getCurrentContext(Item.BIOASSAYSET).setId(bioAssaySetId);
			PluginConfigurationRequest request = j.configure(null);
			request.setParameterValue("source", bioAssaySet);
			request.setParameterValue("childName", childName);
			request.setParameterValue("minIntensity", minIntensity);
			request.setParameterValue("lowExclude", lowExclude);
			request.setParameterValue("highExclude", highExclude);
			request.setParameterValue("blockGroup", blockGroup);

			TestJob.write_request_information(request.getRequestInformation());
			
			PluginResponse response = request.invoke();
			if (response.getStatus() == Response.Status.DONE)
			{
				response.saveParameters(dc);
				dc.saveItem(j);
				dc.commit();
			}
			else
			{
				throw new BaseException(response.getMessage(), response.getErrorList().get(0));
			}
			id = j.getId();
			write("--Create job for Median Ratio Normalization OK");
		}
		catch (Throwable ex)
		{
			write("--Create job for Median Ratio Normalization FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void write(String message)
	{
		System.out.println(message);
	}

}
