/**
	$Id: AbstractRPCService.java 4513 2008-09-11 20:06:36Z jari $

	Copyright (C) 2007 Martin Svensson, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.ws.server;

import net.sf.basedb.core.Application;
import net.sf.basedb.core.SessionControl;

import javax.activation.DataHandler;
import javax.activation.DataSource;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.OMNamespace;
import org.apache.axiom.om.OMText;
import org.apache.axis2.context.MessageContext;

/**
 	An abstract class for Web service classes to extend from.
 	This contains common methods and fields for WebService classes
 	on the server side in BASE.
  	@author Nicklas, Martin
  	@version 2.5
 */
public abstract class AbstractRPCService 
{
	protected ServicesUtil util = new ServicesUtil();
	
	protected AbstractRPCService()
	{}
	
	/**
	 	Gets the SessionControl for a given session ID.
	  	@param ID Id for which SessionControl should be returned.
	  	@return SessionControl object
	 */
	protected SessionControl getSessionControl(String ID)
	{
		return Application.getSessionControl(ID, getRemoteId());
	}
	
	/**
	 	Gets the remote id to identify connection with.
	  	@return A string with the address to the connected client.
	 */
	protected String getRemoteId()
	{
		MessageContext msgCtx = MessageContext.getCurrentMessageContext();
		return msgCtx.getFrom().getAddress();
	}
	
	/**
	 	Attach a file to an OMElement so it can be sent with webservices.
	 	Following code shows an example of how to get to the file on the client side:
	 	<code>
	 		OMElement response = .....<br>
			OMElement fileElement = response.getFirstElement();<br>
			OMElement dataElement = fileElement.getFirstElement();<br>
			OMText node = (OMText) dataElement.getFirstOMChild();<br>	
			DataHandler dataHandler = (DataHandler) node.getDataHandler();<br>		
			InputStream in = dataHandler.getInputStream();<br>
	 	 </code>
	  	@param dataSource A DataSource to be attached to an OMElement. 
	  	@param elementName The name to call the element by.
	  	@return OMElement where the file is attached to.
	 */
	protected OMElement attachFile(DataSource dataSource, String elementName)
	{
		OMFactory messageFactory = OMAbstractFactory.getOMFactory();
		OMNamespace nameSpace = messageFactory.createOMNamespace(getRemoteId(), elementName);				
		OMElement fileElement = messageFactory.createOMElement(elementName, nameSpace);
		
		DataHandler dataHandler = new DataHandler(dataSource);
		OMText node = messageFactory.createOMText(dataHandler, true);		
		fileElement.addChild(node);
	
		return fileElement;
	}
}
